#!/bin/sh
#
# Repack upstream source removing generated (.mo) files
# and eg/hangman.p6, which has no license and is certainly
# copyrighted
#
# To be called via debian/watch (uscan or uscan --force)
# or
#  sh debian/repack.sh --upstream-version VER FILE

set -e
set -u

usage() {
    cat <<EOF >& 2
Usage: $0 --upstream-version VER FILE

            or

       uscan [--force]
EOF
}

[ "${1:-}" = "--upstream-version" ] \
    && [ -n "${2:-}" ] \
    && [ -n "${3:-}" ] \
    && [ -z "${4:-}" ] \
    || usage

TMPDIR=`mktemp -d -p .`

trap "rm -rf $TMPDIR" INT QUIT 0

VER="$2"
DEB_VER="${VER}.ds1"
UP_VER="${VER}"
UPSTREAM_TAR="$3"
UPSTREAM_DIR=Padre-${UP_VER}
ORIG="../padre_${DEB_VER}.orig.tar.gz"
ORIG_DIR="padre-${DEB_VER}.orig"

if [ -e "$ORIG" ]; then
    echo "$ORIG already exists. Aborting."
    exit 1
fi

echo -n "Expanding upstream source tree..."
tar xzf $UPSTREAM_TAR -C $TMPDIR
echo " done."

# clean generated files
echo "Cleaning upstream sources from generated files..."
for f in `cd $TMPDIR/$UPSTREAM_DIR/share/locale; ls -1 *.mo` ;
do
    echo "  share/locale/$f"
    rm $TMPDIR/$UPSTREAM_DIR/share/locale/$f
done

echo "Remmoving copyrighted files without a license..."
for f in eg/hangman.p6 ;
do
    echo "  $f"
    rm $TMPDIR/$UPSTREAM_DIR/$f
done


mv $TMPDIR/$UPSTREAM_DIR $TMPDIR/$ORIG_DIR

echo -n Repackaging into ${ORIG} ...
tar c -C $TMPDIR $ORIG_DIR | gzip -n -9 > "$ORIG"
echo " done."
